@@ -3,12 +3,18 @@
 """Tests for Fn.py library"""
 
 import unittest
-
-from fn import op, _, F, Stream, iters, underscore
-
 import operator
 import itertools
 
+from sys import version_info
+
+from fn import op, _, F, Stream, iters, underscore, monad
+
+class InstanceChecker(object):
+    if version_info[0] == 2 and version_info[1] <= 6:
+        def assertIsInstance(self, inst, cls):
+            self.assertTrue(isinstance(inst, cls))
+
 class OperatorTestCase(unittest.TestCase):
 
     def test_currying(self):
@@ -511,6 +517,105 @@ class StreamTestCase(unittest.TestCase):
         # 35 elements should be already evaluated
         self.assertEqual(fib.cursor(), 35)
 
+class OptionTestCase(unittest.TestCase, InstanceChecker):
+
+    def test_create_option(self):
+        self.assertIsInstance(monad.Option("A"), monad.Full)
+        self.assertIsInstance(monad.Option(10), monad.Full)
+        self.assertIsInstance(monad.Option(10, lambda x: x > 7), monad.Full)
+        self.assertIsInstance(monad.Option(None), monad.Empty)
+        self.assertIsInstance(monad.Option(False), monad.Empty)
+        self.assertIsInstance(monad.Option(10, lambda x: x > 70), monad.Empty)
+
+    def test_map_filter(self):
+        class Request(dict):
+            def parameter(self, name):
+                return monad.Option(self.get(name, None))
+
+        r = Request(testing="Fixed", empty="   ")
+
+        # full chain
+        self.assertEqual("FIXED", r.parameter("testing")
+                                   .map(operator.methodcaller("strip"))
+                                   .filter(len)
+                                   .map(operator.methodcaller("upper"))
+                                   .getOr(""))
+
+        # breaks on filter
+        self.assertEqual("", r.parameter("empty")
+                              .map(operator.methodcaller("strip"))
+                              .filter(len)
+                              .map(operator.methodcaller("upper"))
+                              .getOr(""))
+
+        # breaks on parameter
+        self.assertEqual("", r.parameter("missed")
+                              .map(operator.methodcaller("strip"))
+                              .filter(len)
+                              .map(operator.methodcaller("upper"))
+                              .getOr(""))
+
+    def test_lazy_orcall(self):
+        def from_mimetype(request):
+            # you can return both value or Option
+            return request.get("mimetype", None)
+
+        def from_extension(request):
+            # you can return both value or Option
+            return monad.Option(request.get("url", None))\
+                        .map(lambda s: s.split(".")[-1])
+
+
+        # extract value from extension
+        r = dict(url="myfile.png")
+        self.assertEqual("PNG", monad.Option(r.get("type", None)) \
+                                     .orCall(from_mimetype, r) \
+                                     .orCall(from_extension, r) \
+                                     .map(operator.methodcaller("upper")) \
+                                     .getOr(""))
+
+        # extract value from mimetype
+        r = dict(url="myfile.svg", mimetype="png")
+        self.assertEqual("PNG", monad.Option(r.get("type", None)) \
+                                     .orCall(from_mimetype, r) \
+                                     .orCall(from_extension, r) \
+                                     .map(operator.methodcaller("upper")) \
+                                     .getOr(""))
+
+        # type is set directly
+        r = dict(url="myfile.jpeg", mimetype="svg", type="png")
+        self.assertEqual("PNG", monad.Option(r.get("type", None)) \
+                                     .orCall(from_mimetype, r) \
+                                     .orCall(from_extension, r) \
+                                     .map(operator.methodcaller("upper")) \
+                                     .getOr(""))
+
+    def test_optionable_decorator(self):
+        class Request(dict):
+            @monad.optionable
+            def parameter(self, name):
+                return self.get(name, None)
+
+        r = Request(testing="Fixed", empty="   ")
+
+        # full chain
+        self.assertEqual("FIXED", r.parameter("testing")
+                                   .map(operator.methodcaller("strip"))
+                                   .filter(len)
+                                   .map(operator.methodcaller("upper"))
+                                   .getOr(""))
+
+    def test_stringify(self):
+        self.assertEqual("Full(10)", str(monad.Full(10)))
+        self.assertEqual("Full(in box!)", str(monad.Full("in box!")))
+        self.assertEqual("Empty()", str(monad.Empty()))
+        self.assertEqual("Empty()", str(monad.Option(None)))
+
+    def test_option_repr(self):
+        self.assertEqual("Full(10)", repr(monad.Full(10)))
+        self.assertEqual("Full(in box!)", repr(monad.Full("in box!")))
+        self.assertEqual("Empty()", repr(monad.Empty()))
+        self.assertEqual("Empty()", repr(monad.Option(None)))
 
 if __name__ == '__main__':
     unittest.main()
